.container {
  position: absolute;
  top: 0;
  left: 0;
  width: 100%;
  height: 100%;
  z-index: 1000;
}

.center {
  position: relative;
  top: 50%;
  left: 50%;
  transform: translate(-50%, -50%);
  -ms-transform: translate(-50%, -50%);
  width: 80%;
  text-align: center;
}

.button {
  display: flex;
  justify-content: space-around;
  text-align: center;
  user-select: none;

  background-color: #f1f1f1;
  color: black;
  font-size: 16px;
  padding: 10px 20px;
  border: none;
  cursor: pointer;
  border-radius: 5px;
  text-align: center;
  width: 200px;
  margin: 10px auto;
}
