import TrameApp from "./TrameApp";
import TrameExec from "./TrameExec";
import TrameGetter from "./TrameGetter";
import TrameLoading from "./TrameLoading";
import TrameReconnect from "./TrameReconnect";
import TrameScript from "./TrameScript";
import TrameStyle from "./TrameStyle";
import TrameTemplate from "./TrameTemplate";
import TrameClientStateChange from "./TrameClientStateChange";
import TrameClientTriggers from "./TrameClientTriggers";
import TrameDeepReactive from "./TrameDeepReactive";
import TrameLifeCycleMonitor from "./TrameLifeCycleMonitor";
import TrameSizeObserver from "./TrameSizeObserver";
import TrameHandler from "./TrameHandler";

export default {
  TrameApp,
  TrameExec,
  TrameGetter,
  TrameLoading,
  TrameReconnect,
  TrameScript,
  TrameStyle,
  TrameTemplate,
  TrameClientStateChange,
  TrameClientTriggers,
  TrameDeepReactive,
  TrameLifeCycleMonitor,
  TrameSizeObserver,
  TrameHandler,
};
